
# Given a matrix representing a hyperplane arrangement and its circuits, 
# get a universal GB for the reciprocal linear space
# -------------  Input:
# L                         a matrix ::Matrix{Int64}
# -------------  Output:
# GB                a Groebner basis ::Vector{Expression}
# y           HC variables of the GB ::Vector{Variable}

function get_Groebner_basis(L)
    
    circs = get_circuits(L)
    d, n = size(L).-(1,1)
    @var y[0:n]
    GB = []
    for circ in circs
        c = nullspace(matrix_space(Oscar.ZZ,size(L[:,circ])...)(L[:,circ]))[2]
        c = [Rational{Int64}(Int(cc)) for cc in c]
        push!(GB,sum([c[i]*prod(y[setdiff(circ,circ[i])]) for i = 1:length(circ)]))
    end
    return GB, y
end

# Solve a linear system given by general matrix A0 on the union of coordinate subspaces
# -------------  Input:
# A0              a (generic) matrix ::Matrix{ComplexF64}
# L                         a matrix ::Matrix{Int64}
# w       a random weight vector ::Transpose{Int64, Vector{Int64}}
# -------------  Output:
# startsols      solutions 

function solve_start_system(A0, L, w)
    
    circs = get_circuits(L)
    d, n = size(A0,1), size(A0,2)-1
    brokencircs = [setdiff(c, c[argmin(w[c])]) for c in circs]

    N = matroid_from_circuits(brokencircs, n+1)
    basesN = bases(N)
    groundset =  collect(1:n+1)

    coords = [setdiff(groundset, b) for b in basesN]
    
    startsols = []
    for i = 1:length(coords)
        xx = zeros(ComplexF64,n+1)
        xx[setdiff(1:n+1,coords[i])] = nullspace(A0[:,setdiff(1:n+1,coords[i])])
        push!(startsols,xx)
    end
    
    return startsols
end

# Solve a linear system given by L on the reciprocal linear space variety
# -------------  Input:
# L                                             a matrix ::Matrix{Int64}
# u                                a general data vector ::Vector{ComplexF64}
# A0 (optional)                       a (generic) matrix ::Matrix{ComplexF64}
# return_boundary    a flag to return boundary solutions ::Boolean
# -------------  Output:
# sols_xcoord    non-boundary solutions to the scattering equations in initial coordinates if return_boundary = false
#                else all solutions in y coordinates

function solve_PS(L, u; A0 = nothing, w = nothing, return_boundary = false)
    
    if A0 === nothing
        d,n = size(L).-(1,1)
        A0 = randn(ComplexF64,d,n+1)
    end

    if w === nothing
        w = get_weight(L)
    end

    GB, y = get_Groebner_basis(L)
    
    startsols = solve_start_system(A0, L, w)
    
    @var t
    H1 = [weight_deformation_for_poly(g, y, w, t) for g in GB]
    H2 = (A0.*(1-t) + randn(ComplexF64)*t*L[2:end,:]*diagm(u))*y
    
    F = System([H1;H2], variables = y, parameters = [t])
    result = HomotopyContinuation.solve(F,startsols, start_parameters = [0], target_parameters = [1]) 
    
    sols = solutions(result)
    nonzero_sols = filter(v -> all(a -> !isapprox(a, 0.0; atol=1e-10, rtol=0), v), sols)
    
    sols_xcoord = []
    for sol in nonzero_sols
        push!(sols_xcoord,find_sols_in_xcoords(L, sol))
    end
    
    rec_deg = length(path_results(result))
    MLdeg = length(sols_xcoord)
    
    println("Number of solutions on the reciprocal space: ", rec_deg, "\tNumber of solutions in the open stratum: ", MLdeg)

    if return_boundary
        sols_boundary = HomotopyContinuation.solutions(result, only_nonsingular = false)
        return result, sols_boundary
    else
        return sols_xcoord
    end
end