# Given polynomial, compute a HC polynomial in an extra variable t that degenerates to a monomial for t -> 0
# -------------  Input:
# poly          a HC polynomial ::Expression
# vars             HC variables ::Vector{Variable}
# weight        a weight vector ::Vector{Int64}
# t                   parameter ::Variable
# -------------  Output:
# fu_HC         a HC polynomial ::Expression 

function weight_deformation_for_poly(poly::Expression, vars::Vector{Variable}, weight::Vector{Int64}, t::Variable)
    
    exponents, coeffs = exponents_coefficients(poly, vars)
    modified_exponents = transpose(exponents) * weight
    leading = maximum(modified_exponents)
    
    new_exponents = vcat(-transpose(modified_exponents.-leading), exponents) 
    n = size(new_exponents)[1]
    m = size(new_exponents)[2]
    
    new_vars = vcat(t, vars)
    
    return sum(i -> coeffs[i]*prod(j -> new_vars[j]^new_exponents[:, i][j], 1:n), 1:m)
end

# Given a matrix, get a random weight vector
# -------------  Input:
# L          a matrix 
# -------------  Output:
# w      a weight vector with distinct integer entries ::Transpose{Int64, Vector{Int64}}

function get_weight(L)
    
    _, n = size(L).-(1,1)
    return shuffle(1:n+1)
    
end

# Given a matrix representing a matroid, get its circuits
# -------------  Input:
# L          a matrix 
# -------------  Output:
#       vector of circuits ::Vector{Vector{Int64}}

function get_circuits(L) 
    return circuits(matroid_from_matrix_columns(matrix_space(Oscar.QQ,size(L)...)(L)))
end

# Given a solution in y coordinates and a matrix defining an arrangement L
# find solution in x coordinates
# -------------  Input:
# L          a matrix which columns are coefficients of the hyperplanes from an arrangement
# y          a vector of solutions on the reciprocal linear space; y_i = 1/l_i(x)
# -------------  Output:
#       solutions to the scattering equations in x coordinates
function find_sols_in_xcoords(L, y)
    At = L[2:end,:]
    b = -L[1,:]
    recip_y = (i -> 1/i).(y)
    
    P = matroid_from_matrix_columns(matrix_space(Oscar.QQ,size(At)...)(At))
    basis = bases(P)[1]
    
    return (transpose(At[:,basis])\((b+recip_y)[basis]))
end

# Given two vectors, check that they are approximately equal up to scaling
# -------------  Input:
# v1          a vector ::Union{Vector{Int64}, Vector{Float64}, Vector{ComplexF64}}
# v2          a vector ::Union{Vector{Int64}, Vector{Float64}, Vector{ComplexF64}}
# -------------  Output:
#       true or false
function are_projectively_equal(v1, v2)
    
    k1 = findall(i -> !isapprox(i, 0.0; atol=1e-10, rtol=0), v1)[1]
    k2 = findall(i -> !isapprox(i, 0.0; atol=1e-10, rtol=0), v2)[1]
    
    w1 = (i -> i/v1[k1]).(v1)
    w2 = (i -> i/v2[k2]).(v2)
    
    return all(i -> isapprox(i, 0.0; atol=1e-10, rtol=0), w1 - w2)
end

# Given a vector (of solutions) of vectors, return unique projective solutions
# -------------  Input:
# v          a vector ::Union{Vector{Vector{Float64}}, Vector{Vector{ComplexF64}}}
# -------------  Output:
#      a vector of unique solutions in projective space a vector ::Union{Vector{Vector{Float64}}, Vector{Vector{ComplexF64}}}
function projectively_unique(v)
    
    for i in collect(1:length(v))
        k = findall(a -> !isapprox(a, 0.0; atol=1e-10, rtol=0), v[i])[1]
        v[i] = (a -> a/v[i][k]).(v[i])
        v[i] = round.(v[i],digits = 10)
    end
    #println(v)
    
    w = []
    i = 1
    while i <= length(v)
        if all(a -> (v[i] != a),w)
            push!(w, v[i])
        end
        i = i + 1
    end
    
    return w
end

